@@ -39,17 +39,23 @@ func isOpenWrt() (ok bool) {
 	return false
 }
 
+func notifyReconfigureSignal(c chan<- os.Signal) {
+	signal.Notify(c, windows.SIGHUP)
+}
+
 func notifyShutdownSignal(c chan<- os.Signal) {
 	// syscall.SIGTERM is processed automatically.  See go doc os/signal,
 	// section Windows.
 	signal.Notify(c, os.Interrupt)
 }
 
+func isReconfigureSignal(sig os.Signal) (ok bool) {
+	return sig == windows.SIGHUP
+}
+
 func isShutdownSignal(sig os.Signal) (ok bool) {
 	switch sig {
-	case
-		os.Interrupt,
-		syscall.SIGTERM:
+	case os.Interrupt, syscall.SIGTERM:
 		return true
 	default:
 		return false
